@@ -3,7 +3,7 @@
 # Programmer friendly subprocess wrapper.
 #
 # Author: Peter Odding <peter@peterodding.com>
-# Last Change: October 4, 2015
+# Last Change: October 6, 2015
 # URL: https://executor.readthedocs.org
 
 """
@@ -49,7 +49,7 @@ import subprocess
 import tempfile
 
 # External dependencies.
-from property_manager import PropertyManager, mutable_property, required_property
+from property_manager import PropertyManager, mutable_property, required_property, writable_property
 
 # Define an alias for Unicode strings that's unambiguous
 # whether we are running under Python 2 or Python 3.
@@ -61,7 +61,7 @@ except NameError:
     unicode = str
 
 # Semi-standard module versioning.
-__version__ = '5.0'
+__version__ = '6.2'
 
 # Initialize a logger.
 logger = logging.getLogger(__name__)
@@ -234,7 +234,7 @@ class ExternalCommand(PropertyManager):
 
     def __init__(self, *command, **options):
         """
-        Construct an :class:`ExternalCommand` object.
+        Initialize an :class:`ExternalCommand` object.
 
         :param command: Any positional arguments are converted to a list and
                         used to set :attr:`command`.
@@ -784,7 +784,7 @@ class ExternalCommand(PropertyManager):
                 kw['stderr'] = self.stderr_stream.fd
             else:
                 kw['stderr'] = subprocess.PIPE
-        # Construct the subprocess object.
+        # Create the subprocess object.
         self.logger.debug("Executing external command: %s", quote(kw['args']))
         self.subprocess = subprocess.Popen(**kw)
         # Synchronously wait for the external command to end?
@@ -832,6 +832,10 @@ class ExternalCommand(PropertyManager):
         """
         Terminate a running process.
 
+        :returns: :data:`True` if the process was terminated, :data:`False`
+                  otherwise (e.g. because :func:`start()` was never called or
+                  the process just finished).
+
         Uses the :func:`subprocess.Popen.terminate()` function. Calls
         :func:`wait()` after terminating the process so that the external
         command's output is loaded and temporary resources are cleaned up. The
@@ -844,6 +848,9 @@ class ExternalCommand(PropertyManager):
             self.logger.debug("Terminating external command: %s", quote(self.command_line))
             self.subprocess.terminate()
             self.wait(check=False)
+            return True
+        else:
+            return False
 
     def load_output(self):
         """
@@ -1058,29 +1065,41 @@ class ExternalCommandFailed(Exception):
 
     This exception is raised by :func:`execute()`,
     :func:`~ExternalCommand.start()` and :func:`~ExternalCommand.wait()` when
-    an external command exits with a nonzero status code. Exposes the following
-    attributes:
-
-    .. attribute:: command
-
-       The :class:`ExternalCommand` object.
+    an external command exits with a nonzero status code.
     """
 
-    def __init__(self, command):
+    def __init__(self, command, pool=None):
         """
         Initialize an :class:`ExternalCommandFailed` object.
 
-        :param command: The :class:`ExternalCommand` object.
+        :param command: The :class:`ExternalCommand` object that triggered the
+                        exception.
+        :param pool: The :class:`.CommandPool` object that triggered the
+                     exception (optional).
         """
-        self.command = command
         super(ExternalCommandFailed, self).__init__(command.error_message)
+        self.command = command
+        self.pool = None
+
+    @writable_property(usage_notes=False)
+    def command(self):
+        """The :class:`ExternalCommand` object that triggered the exception."""
+
+    @writable_property(usage_notes=False)
+    def pool(self):
+        """
+        The :class:`.CommandPool` object that triggered the exception.
+
+        This property will be :data:`None` when the exception wasn't raised
+        from a command pool.
+        """
 
     @property
     def returncode(self):
-        """Shortcut for :attr:`ExternalCommand.returncode`."""
+        """Shortcut for the external command's :attr:`~ExternalCommand.returncode`."""
         return self.command.returncode
 
     @property
     def error_message(self):
-        """Shortcut for :attr:`ExternalCommand.error_message`."""
+        """Shortcut for the external command's :attr:`~ExternalCommand.error_message`."""
         return self.command.error_message
