@@ -1,7 +1,7 @@
 # Automated tests for the `executor' module.
 #
 # Author: Peter Odding <peter@peterodding.com>
-# Last Change: October 4, 2015
+# Last Change: October 6, 2015
 # URL: https://executor.readthedocs.org
 
 """Automated tests for the `executor` package."""
@@ -29,7 +29,7 @@ from executor import (
     quote,
     which,
 )
-from executor.concurrent import CommandPool
+from executor.concurrent import CommandPool, CommandPoolFailed
 from executor.contexts import LocalContext, RemoteContext
 from executor.ssh.client import (
     DEFAULT_CONNECT_TIMEOUT,
@@ -328,30 +328,85 @@ class ExecutorTestCase(unittest.TestCase):
         assert all(cmd.returncode == 0 for cmd in results.values())
         assert timer.elapsed_time < (num_commands * sleep_time)
 
+    def test_command_pool_resumable(self):
+        """Make sure command pools can be resumed after raising exceptions."""
+        pool = CommandPool()
+        # Prepare two commands that will both raise an exception.
+        c1 = ExternalCommand('exit 1', check=True)
+        c2 = ExternalCommand('exit 42', check=True)
+        # Add the commands to the pool and start them.
+        pool.add(c1)
+        pool.add(c2)
+        pool.spawn()
+        # Wait for both commands to finish.
+        while not pool.is_finished:
+            time.sleep(0.1)
+        # The first call to collect() should raise an exception about `exit 1'.
+        e1 = intercept(pool.collect)
+        assert e1.command is c1
+        # The second call to collect() should raise an exception about `exit 42'.
+        e2 = intercept(pool.collect)
+        assert e2.command is c2
+
+    def test_command_pool_termination(self):
+        """Make sure command pools can be terminated on failure."""
+        pool = CommandPool()
+        # Include a command that just sleeps for a minute.
+        sleep_cmd = ExternalCommand('sleep 60')
+        pool.add(sleep_cmd)
+        # Include a command that immediately exits with a nonzero return code.
+        pool.add(ExternalCommand('exit 1', check=True))
+        # Start the command pool and terminate it as soon as the control flow
+        # returns to us (because `exit 1' causes an exception to be raised).
+        try:
+            pool.run()
+            assert False, "Assumed CommandPool.run() to raise ExternalCommandFailed!"
+        except ExternalCommandFailed as e:
+            # Make sure the exception was properly tagged.
+            assert e.pool == pool
+        # Make sure the sleep command was terminated.
+        assert sleep_cmd.is_terminated
+
+    def test_command_pool_delay_checks(self):
+        """Make sure command pools can delay error checking until all commands have finished."""
+        pool = CommandPool(delay_checks=True)
+        # Include a command that fails immediately.
+        pool.add(ExternalCommand('exit 1', check=True))
+        # Include some commands that just sleep for a while.
+        pool.add(ExternalCommand('sleep 1', check=True))
+        pool.add(ExternalCommand('sleep 2', check=True))
+        pool.add(ExternalCommand('sleep 3', check=True))
+        # Make sure the expected exception is raised.
+        self.assertRaises(CommandPoolFailed, pool.run)
+        # Make sure all commands were started.
+        assert all(cmd.was_started for id, cmd in pool.commands)
+        # Make sure all commands finished.
+        assert all(cmd.is_finished for id, cmd in pool.commands)
+
     def test_command_pool_logs_directory(self):
         """Make sure command pools can log output of commands in a directory."""
-        directory = tempfile.mkdtemp()
+        root_directory = tempfile.mkdtemp()
+        sub_directory = os.path.join(root_directory, 'does-not-exist-yet')
         identifiers = [1, 2, 3, 4, 5]
         try:
-            pool = CommandPool(concurrency=5, logs_directory=directory)
+            pool = CommandPool(concurrency=5, logs_directory=sub_directory)
             for i in identifiers:
                 pool.add(identifier=i, command=ExternalCommand('echo %i' % i))
             pool.run()
-            files = os.listdir(directory)
+            files = os.listdir(sub_directory)
             assert sorted(files) == sorted(['%s.log' % i for i in identifiers])
             for filename in files:
-                with open(os.path.join(directory, filename)) as handle:
+                with open(os.path.join(sub_directory, filename)) as handle:
                     contents = handle.read()
                 assert filename == ('%s.log' % contents.strip())
         finally:
-            shutil.rmtree(directory)
+            shutil.rmtree(root_directory)
 
     def test_ssh_command_lines(self):
         """Make sure SSH client command lines are correctly generated."""
         # Construct a remote command using as much defaults as possible and
         # validate the resulting SSH client program command line.
         cmd = RemoteCommand('localhost', 'true', ssh_user='some-random-user')
-        cmd.logger.debug("Command line: %s", cmd.command_line)
         for token in (
                 'ssh', '-o', 'BatchMode=yes',
                        '-o', 'ConnectTimeout=%i' % DEFAULT_CONNECT_TIMEOUT,
@@ -480,6 +535,16 @@ class ExecutorTestCase(unittest.TestCase):
         assert context.capture('hostname') == socket.gethostname()
 
 
+def intercept(func, *args, **kw):
+    """Intercept and return a raised exception."""
+    try:
+        func(*args, **kw)
+    except Exception as e:
+        return e
+    else:
+        assert False, "Expected exception to be raised, but nothing happened! :-s"
+
+
 def tokenize_command_line(cmd):
     """Tokenize a command line string into a list of strings."""
     return sum(map(shlex.split, cmd.command_line), [])
